#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
View

A set of helper to create and instantiate view items
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import Here any required modules for this module.

__all__ = ['ViewFactory', 'ViewComponent']

# Project dependences
from sleipnir.core.singleton import Singleton
from sleipnir.components.loaders import LoaderManager
from sleipnir.components.manager import ComponentManager
from sleipnir.components.components import Component, implements

# Subpackage dependences
from .interfaces.view import IView
from .constants import __views_dir__


#pylint: disable-msg=R0903
class Views(Singleton, ComponentManager):
    """
    A Helper to load and instantiate 'IView' components which
    implements required ifaces
    """

    def __init__(self):
        super(Views, self).__init__()
        python_loader = lambda x: x.__class__.__name__.startswith("Py")
        LoaderManager(self).load(__views_dir__, lfilter=python_loader)

    def get(self, item_type):
        """Get a view component instance that implements 'item_type'"""
        views = self.available(IView)
        item_type = item_type.lower()
        return self[views[0]] if len(views) == 1 else None


#pylint: disable-msg=R0921
class ViewComponent(Component):
    """Base class from which derive Component factories"""

    abstract = True
    implements(IView)

    class QViewInstance(object):
        """Internal representation of a View"""

        __slots__ = ('window', 'presenter',)

        def __init__(self, window):
            self.window = window
            self.presenter = None

    def __init__(self):
        self._items = {}

    def create(self, window, view=None, facade=None, share_ctx=False):
        """
        Get a valid item presenter for window or create one if a
        previous one not exists
        """
        # if exists return current view
        instance = self._items.setdefault(window, self.QViewInstance(window))
        if instance.presenter is not None:
            return instance.presenter

        # Creare required elements and return a valid view
        context = self._create_context(window, share_ctx)
        cmpnent = self._create_component(window)
        view = view or self._create_item(cmpnent, context)
        instance.presenter = self._create_presenter(view, facade)
        return instance.presenter

    def remove(self, item):
        """Make a view item private. Destroy register stuff"""
        for key, value in self._items.iteritems:
            if id(value.item) == item:
                del self._items[key]
            break

    #pylint: disable-msg=R0201
    def _create_item(self, component, context):
        """Create an item based on component and context"""
        return component.create(context)

    def _create_component(self, window, source=None):
        """Creates a item factory to instantiate"""
        raise NotImplementedError

    def _create_context(self, window, share_context):
        """Create a valid context for items"""
        raise NotImplementedError

    def _create_presenter(self, item, facade):
        """Create a valid presenter for this view"""
        raise NotImplementedError



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
DBus

Dbus connection and wrrapper classes
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
# pylint:disable-msg=W0406
import dbus
import dbus.service
import dbus.mainloop.glib
import functools

__all__ = ['RPCObject', 'Connection', 'rpc_method']

# Project requirements
from sleipnir.core.singleton import Singleton

# local submodule requirements
from . import constants


#pylint: disable-msg=E1101
class Connection(Singleton):
    """A Wrapper around session dbus"""

    def __init__(self, name=constants.__appname__, instance_name=None):
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
        self._bus = dbus.SessionBus()
        self._bus_name = name
        self._bus_instance = "".join(("/", instance_name))
        self._remote = None

    def __register(self):
        """
        Try to register get a previously registered instance. if fails
        take ownership of bus for name
        """
        if not self._remote:
            if self._bus_instance:
                try:
                    self._remote = self._bus.get_object( \
                        self._bus_name,                  \
                        self._bus_instance)
                    return
                except dbus.DBusException:
                    pass
            self._remote = dbus.service.BusName(self._bus_name, self._bus)

    @property
    def session_bus(self):
        """Get session bus"""
        return self._bus

    @property
    def bus_name(self):
        """Get registered service name"""
        return self._bus_name

    @property
    def bus_object_name(self):
        """Get registered service instance"""
        return self._bus_instance

    def is_client(self):
        """Check if a connection to a remote was established"""
        self.__register()
        return type(self._remote) not in (dbus.service.BusName,)

    def is_server(self):
        """True if we are the implementation server"""
        return not self.is_client

    def get_interface(self, interface):
        """Get a remote interface to 'interface'"""
        assert self.is_client()
        return dbus.Interface(self._remote, interface)


#pylint: disable-msg=R0903, R0923
class RPCObject(dbus.service.Object):
    """A weapper around an inplemented service"""

    def __init__(self, connection, wrapper):
        # export this object to dbus
        session_bus = connection.session_bus
        session_nam = connection.bus_object_name
        dbus.service.Object.__init__(self, session_bus, session_nam)
        self._wrapper = wrapper
        self._bus = connection

        # To export a Qt signal as a DBus-signal, you need to connect
        # it to a method in this class.  The method MUST have the
        # signal annotation, so python-dbus will export it as a
        # dbus-signal

    @property
    def wrapper(self):
        """Gets wrapper object"""
        return self._wrapper

#pylint: disable-msg=C0103
rpc_method = functools.partial(dbus.service.method, out_signature='')



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""View problem Interfaces"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules.

__all__ = ['IView', ]

# Project Dependences
from sleipnir.components.entrypoint import Interface


class IView(Interface):
    """Base interface required by view components"""

    @property
    def item(self):
        """Returs item (view) implemented by component"""
        raise NotImplementedError

    def create(self, window, share_ctx):
        """
        Get a valid item view for window or create one if a
        previous one not exists
        """
        raise NotImplementedError

    def remove(self, item):
        """Make a view item private. Destroy register stuff"""
        raise NotImplementedError



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Application

Main Application Component Manager
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
import os
import sys

__all__ = ['Application']

# Project requirements
from sleipnir.components.components import implements, Component
from sleipnir.shell.interfaces import application

# local submodule requirements
from .import constants
#pylint: disable-msg=W0611
from .import signals


#pylint: disable-msg=R0201,W0232
class Application(Component):
    """ Main Handset Component"""

    implements(application.IApplication)

    opt_table = [
        ["", "--target", "", "Override default target", "store", "target"],
        ["", "--screen",  0, "Create window in SCREEN", "store", "screen"],

        ["", "--new_window", False,                                       \
               "Force creation of a new window", "store_true", "new"],
    ]

    @property
    def name(self):
        """Applications's shotname"""
        return constants.__modname__

    @property
    def title(self):
        """Application's name"""
        return constants.__title__

    @property
    def summary(self):
        """Application's summary"""
        return constants.__summary__

    @property
    def version(self):
        """Application's version"""
        return constants.__version__

    @property
    def copyright(self):
        """Author's details"""
        return constants.__copyright__

    @property
    def doap(self):
        """Returns description of project"""
        return """\n%s %s\n%s\n""" % self.title, self.version, self.copyright

    @property
    def runnable(self):
        """Check if application is runnable under enviroment"""
        if os.environ.get('SHLVL', 1) > 1 or 'DISPLAY' not in os.environ:
            return application.IApplication.SUGGESTED
        return application.IApplication.VALID

    def usage(self, argv):
        """Show command usage when invoked from command line"""

    #pylint: disable-msg=W0613
    def help(self, argv):
        """Returns application OPT Table"""
        return self.opt_table[:]

    def help_commands(self, command=None):
        """
        Returns help info for command parameter or a list of valid
        commands if no command is present
        """

    #pylint: disable-msg=W0613
    def commands(self, name=None):
        """Get a list of available commands for this backend"""
        return []

    def run(self, args, **kwargs):
        "Invoke application"
        # imports
        from .shell import QShell

        # handle target and application args
        appkey = constants.__appname__
        parser = self.compmgr.parser
        parser.add_child_options(appkey, self.opt_table)
        parser.parse(args, parser=appkey, recursive=-1)

        # create shell
        shell = QShell.get_instance(name=constants.__modname__.capitalize())

        # Enter Main loop
        sys.exit(shell.run(parser.remain_args, dict(parser.options)))



#!/usr/bin/python
# Thomas Perl <thp@gpodder.org>, 2009-01-03

"""
Summary

Text-based visual translation completeness summary

Usage: make statistics | python summary.py

"""

import re
import sys
import math


#pylint: disable-msg=R0913
class Language(object):
    """Language calculator"""
    def __init__(self, code, updated, translated, fuzzy, untranslated):
        self.code = code
        self.updated = updated
        self.translated = int(translated)
        self.fuzzy = int(fuzzy)
        self.untranslated = int(untranslated)

    def __get_ratio(self, value):
        "get ratio"
        return value / float(self.translated + self.fuzzy + self.untranslated)

    def get_translated_ratio(self):
        "translated strings ratio"
        return self.__get_ratio(float(self.translated))

    def get_fuzzy_ratio(self):
        "fuzzy strings ratio"
        return self.__get_ratio(float(self.fuzzy))

    def get_untranslated_ratio(self):
        "untranslated strings ratio"
        return self.__get_ratio(float(self.untranslated))

    def __cmp__(self, other):
        return cmp(self.get_translated_ratio(), other.get_translated_ratio())

if __name__ == '__main__':
    WIDTH = 40
    LANGUAGES = []
    for line in sys.stdin:
        match = re.match(
            '^(..)\.po \(([^)]*)\): ('
            '(\d+) translated message[s]?)?'
            '(, (\d+) fuzzy translation[s]?)?'
            '(, (\d+) untranslated message[s]?)?\.', line).groups()

        LANGUAGES.append(Language(
                match[0], match[1], match[3] or '0',
                match[5] or '0',
                match[7] or '0')
        )

        sys.stdout.write("\n")
        sys.stdout.write(" " * 20 + "--== Sleipnir translation summary == --")
        sys.stdout.write("\n")

    for language in sorted(LANGUAGES):
        tc = '#' * (int(math.floor(WIDTH * language.get_translated_ratio())))
        fc = '~' * (int(math.floor(WIDTH * language.get_fuzzy_ratio())))
        uc = ' ' * (WIDTH - len(tc) - len(fc))

        sys.stdout.write(
            ' %s (%s) [%s%s%s] -- %3.0f %% translated' %
            (language.code,
             language.updated,
             tc, fc, uc,
             language.get_translated_ratio() * 100)
            )

    sys.stdout.write("\n")
    sys.stdout.write('\tTotal translations: %d' % len(LANGUAGES))
    sys.stdout.write("\n")



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
QComponent

Main Application Component Manager
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules

# QML required deps
from PySide.QtCore import QObject, Slot
from PySide.QtDeclarative import QDeclarativeComponent, QDeclarativeContext

__all__ = ['ComponentFactory', 'QComponent']

# Project requirements

# local submodule requirements
from .view import ViewComponent


#pylint: disable-msg=R0904,W0223
class QViewComponent(QObject, ViewComponent):
    """Abstracft implementation targeted to PySide views"""

    abstract = True

    # pylint: disable-msg=C0103,W0232,R0903
    class __metaclass__(type(QObject), type(ViewComponent)):
        def __new__(cls, name, bases, dct):
            new_class = type(QObject).__new__(cls, name, bases, dct)
            if name not in ('Component', ) and 'abstract' not in dct:
                type(ViewComponent).__register_component__(new_class)
            return new_class

    def __init__(self):
        QObject.__init__(self)
        ViewComponent.__init__(self)

    def create(self, window, view=None, facade=None, share_ctx=False):
        connect = window in self._items
        item = ViewComponent.create(self, window, view, facade, share_ctx)
        if connect is True:
            item.close.connect(self.remove)
        return item

    @Slot(object)
    def remove(self, item):
        ViewComponent.remove(self, item)

    def _create_component(self, window, source=None):
        return QDeclarativeComponent(window.engine(), source)

    def _create_context(self, window, share_context):
        context = window.engine().rootContext()
        if share_context is False:
            context = QDeclarativeContext(context)
        return context


#pylint: disable-msg=E1101,E0102
class QViewPresenter(QObject):
    """Main QDashBoard Presenter"""

    #pylint: disable-msg=R0903
    class FacadeStub(object):
        """Simple stub for testing double purposes"""

    def __init__(self, window, facade=None):
        QObject.__init__(self)
        self._facade = facade
        self._window = window
        self._window.setProperty("presenter", self)

    @property
    def window(self):
        """Get window to be handle by this presenter"""
        return self._window

    @property
    def facade(self):
        """Set facade to be handle by this presenter"""
        self._facade = self._facade or self.FacadeStub()
        return self._facade

    @facade.setter
    def facade(self, value):
        """Set facade according to value"""
        self._facade = value



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
QWindow

Main Application Component Manager
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
import os

# QML required deps
from PySide.QtCore import Qt
from PySide.QtDeclarative import QDeclarativeView

__all__ = ['QWindowFactory']

# Project requirements
from sleipnir.shell.platform import Platform

# local submodule requirements


class WindowError(Exception):
    """Main Window exception"""


#pylint: disable-msg=W0232,E0602,R0903
class QViewStatus(enum):
    """View status indicators"""
    STORED, DELETED, INSERT, REMOVE = xrange(4)


#pylint: disable-msg=W0232,E0602,R0903
class QWindowOrientaion(enum):
    """Window orientation indicators"""
    LOCK_PORTRAIT, LOCK_LANDSCAPE, AUTO = xrange(128, 131)


#pylint: disable-msg=R0904
class QWindow(QDeclarativeView):
    """Main Window representation"""

    def __init__(self, presenter, platform=None, parent=None):
        QDeclarativeView.__init__(self, parent)
        self._platform = platform or Platform()
        self._presenter = presenter

    @property
    def platform(self):
        """Peek selected platform"""
        return self._platform

    def lookup(self, object_id):
        """Lookup for object_id window hierachy"""
        root_object = self.rootObject()
        if root_object is not None:
            return root_object.findChild(object_id)

    def set_orientation(self, value):
        """Set screen orientation"""
        if value not in QWindowOrientation:
            raise WindowError("unknown orientation value %d", value)
        self.setAttribute(value, True)


class QHandsetWindow(QWindow):
    """ A handset based declarative view"""

    def __init__(self, presenter=None, platform=None, parent=None):
        presenter = presenter or QFremabtlePresenter(self)
        QWindow.__init__(self, presenter, parent)

        # Omit Frames
        self.setAttribute(Qt.FramelessWindowHint)
        # Save some cycles
        self.setAttribure(Qt.WA_NoSystemBackground)
        # Make window traslucent
        self.setResizeMode(QDeclarativeView.SizeRootObjectToView)
        self.setAttribute(Qt.WA_TranslucentBackground)

        # Set graphics capabilities...
        fmt = QGLFormat.defaultFormat()
        fmt.setDirectRendering(True)
        fmt.setDoubleBuffer(True)
        fmt.setSampleBuffers(False)

        # ... and replace viewport
        glw = QGLWidget(fmt)
        glw.setAutoFillBackground(False)
        self.setViewport(glw)

        # Experimental Qt Mobility packages are installed in /opt
        for qtm in ('qtm11', 'qtm12'):
            path = os.path.join(os.sep, 'opt', qtm, 'imports')
            engine.addImportPath(path)


class QDesktopWindow(QWindow):
    """Generic Desktop view"""
    def __init__(self, presenter=None, platform=None, parent=None):
        presenter = presenter or QDesktopPresenter(self)
        QWindow.__init__(self, presenter, parent)



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Presenters

Passive View pattern implementations
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
import os


# QML required deps
from PySide.QtCore import QObject, Slot
from PySide.QtDeclarative import QDeclarativeView

__all__ = ['QWindowFactory']

# Project requirements
from sleipnir.core.singleton import Singleton
from sleipnir.shell.platform import Platform

# local submodule requirements
from .windows import QDesktopWindow, QHandsetWindow
from .controller import QApplicationController
from .helpers import QProxy
from .constants import QML


#pylint: disable-msg=R0903
class QWindowFactory(Singleton):
    """
    A Application Window factory based on target platform

    Each application window is composed of a view, and a helper,
    approaching to TemplateView pattern from [PEAA03]
    """
    def __init__(self):
        self._backends = {
            "desktop":   (QDesktopWindow,   QDesktopPresenter,),
            "handset": (QHandsetWindow, QHandsetPresenter,),
        }

    def create(self, service, target):
        """Create a valid window for this platform"""
        platform = Platform(target)
        platform_name = platform.real.name
        if platform_name in self._backends:
            window, presenter = self._backends[platform_name]
            presenter = presenter(service)
            presenter.set_window(window(presenter, platform))
            return presenter
        raise AttributeError("'%s'not available" % platform_name.capitalize())


#pylint: disable-msg=R0904
class QWindowPresenter(QObject):
    """Base Controller for Sleipnir Window"""

    # pylint: disable-msg=W0142
    class QWindowController(QApplicationController):
        """Custom Application Controller for QWindowPresenter"""

        #pylint: disable-msg=W0232, R0903
        class QWindowStatus(QApplicationController.QApplicationStatus):
            """Custom valid application controller status"""
            CUSTOM = len(QApplicationController.QApplicationStatus)

        FLOW = []

        def __init__(self, profiles=None):
            QApplicationController.__init__(self, profiles)
            [self.register(*flow, profile=None) for flow in self.FLOW]

        def dispatch(self, event, state):
            state = self.QWindowStatus(state)
            return QApplicationController.dispatch(self, event, state)

        #pylint: disable-msg=W0221
        def register(self, event, state, *args, **kwrgs):
            state = self.QWindowStatus(state)
            QApplicationController.register(self, event, state, *args, **kwrgs)

    def __init__(self, service, window=None):
        super(QWindowPresenter, self).__init__()
        #  app controller stuff
        self._app_controller = None
        # service stuff
        self._service = service
        # window stuff
        self._window = None
        self._bridge = None
        self.set_window(window)

    def __getattr__(self, value):
        if self._window:
            return getattr(self._window, value)
        raise AttributeError(value)

    def __del__(self):
        self._window.close()

    def set_window(self, window):
        """Set window to be handle by this presenter"""
        assert not self._window
        if window is not None:
            self._window = window
            self._window.statusChanged.connect(self.ready)
            # Now load source
            self._window.setSource(self.source)

    @property
    def source(self):
        """Get url for qml resource file"""
        return QML("window", self._window.platform.name)

    @Slot(int)
    def ready(self, status=QDeclarativeView.Null):
        """connect bridge to presenter variant prop in root object"""
        if not self._bridge and status == QDeclarativeView.Ready:
            self._bridge = QProxy(self._window.rootObject())
            self._bridge.presenter = self

    @Slot()
    def quit(self):
        """Quit window"""
        self._service.remove_window(self)

    @Slot(object)
    def screen(self, desktop):
        """Get screen number for view"""
        return desktop.screenNumber(self._window)

    @Slot(str)
    def show(self, profiles):
        """Show view"""
        # check that app has been initiated.
        if not self._app_controller:

            # This line register all valid BaseProfiles into App controller
            ctrl = self._app_controller = self.QWindowController(profiles)
            action, view_component, move = ctrl.dispatch(None, "STARTING")
            assert self.status() == QDeclarativeView.Ready

            # create or get a previously created view
            presenter = view_component.create(self._window, facade=None)

            # invoke action:
            if callable(action):
                action(window, presenter, "STARTING")
            # FIXME: Signal or call directly
            # FIXME: Use a valid Facade
            if hasattr(self._bridge, "navigate_to"):
                self._bridge.navigate_to(presenter.window, move)

    @Slot()
    def switch(self):
        """Call to a task manager"""
        raise NotImplementedError


class QHandsetPresenter(QWindowPresenter):
    """A handset based helper"""

    DBUS_SWITCHER_EVENT = "/com/nokia/hildon_desktop " \
        "com.nokia.hildon_desktop.exit_app_view"

    @Slot()
    def switch(self):
        """Send a switch event or minimize"""
        os.system('dbus-send' + self.DBUS_EXIT_EVENT)

    def show(self, profiles):
        """Show view"""
        QWindowPresenter.show(self, profiles)
        self.showFullScreen()


class QDesktopPresenter(QWindowPresenter):
    """Desktop presenter"""

    @Slot()
    def switch(self):
        """Send a switch event or minimize"""
        self._window.showMinimized()

    def show(self, profiles):
        """Show view"""
        QWindowPresenter.show(self, profiles)
        self._window.show()



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Signals

Signals factory for PySide
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
import PySide.QtCore

__all__ = []

# Project requirements
from sleipnir.shell.signals import SignalFactory

# local submodule requirements


#pylint:disable-msg=W0232
class PySideSignal(SignalFactory):
    """PySide based signal factory"""

    @classmethod
    def create(cls, *args, **kwargs):
        """Creates a Signal"""
        return PySide.QtCore.Signal(*args, **kwargs)

    @classmethod
    def platform(cls):
        """Get platform technology"""
        return "PySide.QtCore"



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Shell

Main QML Shell instance
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
import sys

# QML required deps
from PySide.QtGui import QApplication

__all__ = ['QShell']

# Project requirements
from sleipnir.shell.shell import Shell

# local submodule requirements
from .lrpc import Connection, rpc_method, RPCObject
from .presenters import QWindowFactory
from .import constants


#pylint: disable-msg=E1101
class QShell(Shell):
    """Main Handset shell"""

    SHELL_DAEMON = "__Daemon"
    SHELL_WINDOWS = "__Windows"

    # pylint: disable-msg=R0923
    class RPC(RPCObject):
        """Wraps a Shell instance inside a RPC object"""

        @rpc_method(constants.__appname__, in_signature='biss')
        def create_window(self, new, screen, target, profiles):
            """creates a window"""
            window = self.wrapper.create_window(new, screen, target)
            window.show(profiles)

    def __init__(self, name="Handset"):
        # Build parent classes
        Shell.__init__(self, name)

        # First instance ? => All done
        self._conn = Connection.get_instance(instance_name="Shell")
        if self._conn.is_client():
            return

        # Add an accessor for dbus daemon
        self.register(self.SHELL_DAEMON, self.RPC(self._conn, self))

        # Create Qt application and the QDeclarative view
        QApplication.setGraphicsSystem("raster")
        self._app = QApplication(sys.argv)
        self._app.setApplicationName(name)

        # Add an accessor for windows
        self.register(self.SHELL_WINDOWS, [])

    @property
    def windows(self):
        """Get list of windows associated to shell"""
        return self.objects[self.SHELL_WINDOWS]

    def create_window(self, new=True, screen=None, target=None):
        """Create a window in screen"""
        desktop = self._app.desktop()
        wscreen = screen or desktop.primaryScreen()
        wscreen = desktop.screenCount() - 1 if screen == -1 else wscreen

        # create window and move to screen wscreen
        loopfilter = lambda x: x.screen(desktop) == wscreen
        wviews = [window for window in self.windows if loopfilter(window)]
        window = wviews[0] if len(wviews) > 0 else None

        # not found. create one
        if any((new, not window)):
            # create window
            factory = QWindowFactory.get_instance()
            window = factory.create(self, target)
            self.objects[self.SHELL_WINDOWS].append(window)
            # move to wscreen
            geometry = desktop.screenGeometry(wscreen)
            window.move(window.x() + geometry.x(), window.y() + geometry.y())
        return window

    def remove_window(self, window):
        """Remove window from active windows"""
        self.windows.remove(window)
        del window
        if len(self.windows) == 0:
            self._app.aboutToQuit.emit()
            self._app.quit()

    #pylint: disable-msg=W0613
    def run(self, args, opt):
        "Invoke application"
        # process options
        new, screen, target = opt['new'], int(opt['screen']), opt['target']
        if self._conn.is_client():
            iface = self._conn.get_interface(constants.__appname__)
            iface.create_window(new, screen, target, opt['profiles'])
            return 0
        window = self.create_window(new, screen, target)
        window.show(opt['profiles'])
        return self._app.exec_()



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Helper

QML Helper functions
"""

from __future__ import absolute_import

__author__  = "Carlos Martín"
__license__ = "See LICENSE file for details"

# Import here any required modules
import sys

# QML required deps
from PySide.QtCore import QObject, Signal, Property

__all__ = ['qproperty', 'Generator', 'Proxy', ]

# Project requirements

# local submodule requirements


#pylint: disable-msg=C0103,W0142,W0212
class qproperty(property):
    """A property like class decorator for QProperty"""
    def __init__(self, qtype, fget=None, *args, **kwrgs):
        def __clean_kw(kwargs, invalid=("no_notify", "notify",)):
            """removes unsuported args from kwargs"""
            [kwargs.pop(key, None) for key in invalid]
            return kwargs
        # store args
        self._pkwrgs = __clean_kw(dict(kwrgs))
        self._qkwrgs = kwrgs
        super(qproperty, self).__init__(fget, *args, **self._pkwrgs)
        # store qtype
        self._qtype = qtype
        self.__update(fget.__name__ if fget else None)

    def __get__(self, instance, stype=None):
        return getattr(instance, self._qname)

    def __set__(self, instance, value):
        setattr(instance, self._qname, value)
        getattr(instance, self._sname).emit()

    def __call__(self, fget):
        self.__update(fget.__name__)
        cls_ns = sys._getframe(1).f_locals
        cls_ns[self._qname] = Property(self._qtype, fget, **self._pkwrgs)
        return qproperty(self._qtype, fget, **self._qkwrgs)

    #pylint: disable-msg=W0201
    def __update(self, name):
        """Update property, qml property and signal notify"""
        if name is not None:
            self._pname = name
            self._qname = "qml_" + name
            self._sname = name + "_signal"

    def setter(self, fset):
        """Detter decorator"""
        cls_ns = sys._getframe(1).f_locals
        if not self._qkwrgs.pop("no_notify", False):
            cls_ns[self._sname] = self._qkwrgs.setdefault("notify", Signal())
        cls_ns[self._qname] = Property(        \
            self._qtype, self.fget, fset, **self._qkwrgs)
        return qproperty(self._qtype, self.fget, fset, **self._pkwrgs)


def Bridge(*defs, **values):
    """A QObject Generator Class"""

    #pylint: disable-msg=R0904
    class PropertyBridge(QObject):
        """Autogenerated Class"""

        def __init__(self):
            QObject.__init__(self)
            for key, val in defs:
                setattr(self, '_' + key, values.get(key, val()))

        def __repr__(self):
            name = values.get('name', 'QObject')
            func = lambda x: '%s=%r' % (x, getattr(self, '_' + x))
            vals = (func(key) for key, val in defs)
            return '<%s (%s)>' % (name, ', '.join(vals))

        #pylint: disable-msg=C0111,E0213
        for key, value in defs:
            fnotify = locals()['_nfy_' + key] = Signal()

            def _get(key):
                def f(self):
                    return self.__dict__['_' + key]
                return f

            def _set(key):
                def f(self, value):
                    setattr(self, '_' + key, value)
                    getattr(self, '_nfy_' + key).emit()
                return f

            fset = locals()['_set_' + key] = _set(key)
            fget = locals()['_get_' + key] = _get(key)
            locals()[key] = Property(value, fget, fset, notify=fnotify)

    return PropertyBridge


#pylint: disable-msg=R0903
class QProxy(object):
    """
    Proxy for accessing properties and slots as attributes

    This class acts as a proxy for the object for which it is created,
    and makes property access more Pythonic while still allowing
    access to slots (as member functions).

    Attribute names starting with '_' are not proxied.
    """

    def __init__(self, root):
        metao = root.metaObject()
        props = range(metao.propertyCount())
        self._root = root
        self._props = [metao.property(i).name() for i in props]

    def __getattr__(self, key):
        value = self._root.property(key)
        # No such property, so assume we call a slot
        if value is None and key not in self._props:
            return getattr(self._root, key)
        return value

    def __setattr__(self, key, value):
        if key.startswith('_'):
            object.__setattr__(self, key, value)
        else:
            self._root.setProperty(key, value)



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""Sleipnir Handset frontend constants"""

from __future__ import absolute_import

__author__           = 'Carlos Martín <cmartin@liberalia.net>'
__version__          = '0.0.90'
__date__             = '2011-15-04'
__license__          = 'All rights reserved'

__namespace__        = "sleipnir"
__family__           = "frontends"
__modname__          = "handset"
__appname__          = __namespace__ + '.' + __family__ + '.' + __modname__
__title__            = 'Sleipnir N900 UX'
__release__          = '1'
__summary__          = 'UX experience for Nokia N900 (Handset) and Meego'
__url__              = 'http://sleipnir.liberalia.net/'
__copyright__        = '© 2011 Carlos Martín'

__classifiers__      = [
    'Development Status :: 4 - Beta',
	'License :: Other/Proprietary License',
    "Environment :: Handhelds/PDA's",
    "Intended Audience :: Customer Service",
    "Operating System :: POSIX :: Linux",
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    "Topic :: Office/Business :: Scheduling",
    ]

__long_description__ = """\
Add Here a a description to this package
"""

__requires__       = [
    __namespace__ + '.components >= 0.1.90',
    __namespace__ + '.core       >= 0.1.90',
    __namespace__ + '.shell      >= 0.0.93',
    ]
__tests_requires__ = [
    __namespace__ + '.components >= 0.1.90',
    __namespace__ + '.core       >= 0.1.90',
    __namespace__ + '.shell      >= 0.0.93',
    __namespace__ + '.testing    >= 0.1rc6',
    ]


try:
    from os import path
    from sleipnir.shell import constants
    from sleipnir.shell.constants import System as BaseSystem

    #pylint: disable-msg=E1101
    class System(BaseSystem):
        """Derived class to adjust prefix to frontend package"""

        def __init__(self):
            super(System, self).__init__(__file__, __modname__)

        def get_libdir(self):
            """Get python system libdir"""
            return path.join(self.get_srcdir(),  \
                __namespace__, __family__, self._modname)

        def get_viewdir(self):
            """Get views dir for handset"""
            return path.join(self.get_libdir(), 'views')

        # pylint: disable-msg=R0201
        def get_entry_point(self):
            """Get Sleipnir.Components defined Entry Point"""
            return constants.__entry_point__

    # pylint: disable-msg=E1101
    # Common directories
    __ui_dir__      = System.get_instance().get_uidir()
    __views_dir__   = System.get_instance().get_viewdir()
    __entry_point__ = System.get_instance().get_entry_point()

except ImportError:
    __ui_dir__ = __views_dir__ = None
    __entry_point__ = __namespace__ + ".frontends"


#pylint: disable-msg=C0103
def QML(filename, target):
    """QML locator helper"""
    qmlfile = path.splitext(path.basename(filename))[0] + '.qml'
    for custom in (target, 'common'):
        qmlpath = path.join(__ui_dir__, 'qml', custom, qmlfile)
        if path.exists(qmlpath):
            return qmlpath
    raise LookupError("QML resource at '%s' not found", qmlpath)



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Application Controller

Main Application Component Manager
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules
from odict import odict

__all__ = ['QApplicationController']

# Project requirements
from sleipnir.core.log import log

# local submodule requirements
from .profiles import ProfileFactory, Profiles
from .view import Views


class QApplicationController(object):
    """
    A flow navigation controller. See PEAA03 for details about
    Application Controller Pattern
    """
    #pylint: disable-msg=W0232,R0903,E0602
    class QApplicationStatus(enum):
        """Base states"""
        UNKNOWN, STARTING = xrange(2)

    #pylint: disable-msg=R0903
    class State(object):
        """Memento object"""
        def __init__(self, event, status):
            self.event = event
            self.state = status

    def __init__(self, profiles=None):
        self._history = []
        self._dispatchers = odict()
        factory = ProfileFactory.get_instance()
        factory.create(self, Profiles(profiles))

    def register(self, event, state, action, view, where=None, animation=None):
        """Stores a flow for a profile(where)"""
        dispatcher = self._dispatchers.setdefault(where, {})
        dispatcher = dispatcher.setdefault(event, {})
        if state in dispatcher:
            log.warning("Rewrited flow: (%s, %s, %s)" % (where, event, state))
        dispatcher[state] = (action, view, animation)

    def next(self, event, status):
        """Dispatch an event and saves state in history"""
        self._history.append(self.State(event, status))
        return self.dispatch(event, status)

    def prev(self):
        """
        Discards current event and dispatch previous one. Like a
        back dispatch
        """
        if len(self._history) > 1:
            self._history.pop(-1)
            state = self._history[-1]
            return self.dispatch(state.event, state.status)
        raise EOFError

    def dispatch(self, event, status):
        """Peeks action and command and animation for event and status"""
        log.controller.info("Dispatch (%s,%s)" % (event, status))
        for dispatcher in self._dispatchers.iterkeys():
            dispatcher = self._dispatchers[dispatcher][event]
            state = dispatcher[status]
            if state is not None:
                action, view, animation = state
                if action and not callable(action):
                    raise NotImplementedError
                if type(view) in (str, unicode,):
                    view = Views.get_instance().get(view)
                    dispatcher[status] = (action, view, animation)
                return action, view, animation
        return (None, None, None)

    def clear(self, event=None, status="UNKNOWN"):
        "reset history to event and status"
        self._history = [self.State(event, status)]



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""Clients Flow"""

from __future__ import absolute_import

__author__  = "Carlos Martín"
__license__ = "See LICENSE file for details"

# local submodule requirements
from ..profiles import BaseProfile


#pylint: disable-msg=R0903,W0232
class ClientProfile(BaseProfile):
    """Clients Flow"""
    PROVIDES = "clients"
    REGISTER = [[None, "STARTING", None, "IDashBoard", PROVIDES]]



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
QWindow

Main Application Component Manager
"""

from __future__ import absolute_import

__author__  = "Carlos Martin <cmartin@liberalia.net>"
__license__ = "See LICENSE file for details"

# Import here any required modules

# QML required deps

__all__ = ['QDashBoard']

# Project requirements
from sleipnir.frontends.handset.component import QViewComponent
from sleipnir.frontends.handset.component import QViewPresenter
from sleipnir.frontends.handset.constants import QML

# local submodule requirements


#pylint: disable-msg=W0232,R0903
class QDashBoard(QViewPresenter):
    """Main QDashBoard Presenter"""


class DashBoardFactory(QViewComponent):
    """DashBoard Component"""

    item = "IDashBoard"

    #pylint: disable-msg=R0201
    def _create_component(self, window, source=None):
        source = QML(source or __file__, window.platform.name)
        return QViewComponent._create_component(self, window, source)

    def _create_presenter(self, item=None, facade=None):
        return QDashBoard(item, facade)



#!/usr/bin/env python
# -*- mode: Python; indent-tabs-mode: nil; coding: utf-8  -*-

#
# Copyright 2010, 2011 Carlos Martín
# Copyright 2010, 2011 Universidad de Salamanca
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

"""Main Setup File"""

import os
import re
import sys

if sys.version_info < (2, 6):
    sys.stderr.write("Sleipnir requires Python 2.6 or higher.")
    sys.exit(0)

# Load constants
sys.path.insert(0, 'src')
from sleipnir.frontends.handset import constants

# setuptools build required.
from os.path import normcase as _N
from setuptools import setup, find_packages


# pylint: disable-msg=R0903,E0602
class Files(object):
    """
    Custom Distribution Class to force creation of namespace dirs
    """

    @staticmethod
    def find(walk, suffix, install):
        """Find files that match suffix"""

        _files = []

        for root, dirs, files in os.walk(_N(walk)):
            _files.extend(((_N(install), [os.path.join(root, f)]) \
               for f in files if f.endswith(suffix)))
            for entry in dirs:
                _files += Files.find(
                    os.path.join(walk, entry),
                    suffix,
                    os.path.join(install, entry))
            break
        return _files

    @staticmethod
    def find_ui(walk, suffix, dest):
        """sugar method for qml files"""
        return Files.find(
            walk, suffix,
            os.path.join('share', constants.__appname__, dest))


# Peek author details
AUTHOR, EMAIL = re.match(r'^(.*) <(.*)>$', constants.__author__).groups()


# i18n files
I18N_FILES = Files.find(
    'data/po', '.mo', 'share/locale/' + constants.__appname__
)
# Desktop files
DATA_FILES = [
    (_N('share/pixmaps'), [_N('data/sleipnir.png')]),
    (_N('share/applications'), [_N('data/sleipnir.desktop')]),
]
# UI files and themes
DATA_FILES += Files.find_ui('data/ui', '.qml', 'ui')
DATA_FILES += Files.find_ui('data/ui', '.png', 'ui')

setup(
    author             = AUTHOR,
    author_email       = EMAIL,
    classifiers        = constants.__classifiers__,
    description        = constants.__summary__,
    install_requires   = constants.__requires__,
    license            = constants.__license__,
    long_description   = constants.__long_description__,
    name               = constants.__appname__,
    namespace_packages = [constants.__namespace__],
    package_dir        = {'': 'src'},
    packages           = find_packages(where='src'),
    test_suite         = 'tests',
    tests_require      = [constants.__tests_requires__],
    url                = constants.__url__,
    version            = constants.__version__,
    zip_safe           = False,
    data_files         = DATA_FILES + I18N_FILES,
    entry_points = """
    [%s]
      handset.application = sleipnir.frontends.handset.application:Application
    """ % constants.__entry_point__,
)



"""Sleipninr RCA Flows"""

from os import path
from glob import iglob
from operator import itemgetter
from itertools import imap


def __import(match):
    """Import modules"""
    modules = (path.splitext(path.basename(x)) for x in iglob(match))
    modules = set(imap(itemgetter(0), modules))
    [__import__(mod, globals()) for mod in modules if mod[0] != '_']

# now import flows
__import(path.join(path.dirname(__file__), "./*.py*"))



#!/usr/bin/env python
# -*- mode:python; tab-width: 2; coding: utf-8 -*-

"""
Helper

QML Helper functions
"""

from __future__ import absolute_import

__author__  = "Carlos Martín"
__license__ = "See LICENSE file for details"

# Import here any required modules
import re

from odict import odict
from itertools import ifilter, chain

__all__ = ['ProfileFactory', 'Profiles']

# Project requirements
from sleipnir.core.factory import NamedAbstractFactory
from sleipnir.core.log import log

# local submodule requirements


#pylint: disable-msg=W0232,E0602,R0903
class Actions(enum):
    """Actions enumarator"""
    INSERT, REMOVE = xrange(2)


class Profiles(object):
    """Valid profiles activated for"""
    def __init__(self, profiles=None):
        self._profiles = odict()
        self._proregex = re.compile(r"\+|!")
        self.extend(profiles or ProfileFactory.get_instance().flows())

    def __str__(self):
        return ",".join((profile for profile in self))

    def __contains__(self, value):
        value = self._proregex.sub("", value)
        return value in self._profiles

    def __getitem__(self, value):
        return self._profiles[value]

    def __getattr__(self, value):
        return getattr(self._profiles, value)

    def __iter__(self):
        for key, value in self._profiles.iteritems():
            action = '!' if value == Actions.REMOVE else "+"
            yield("".join((action, key)))

    def __or__(self, other):
        # Order should be self,order. We readd self to override
        # actions broken by other
        return self.__class__(chain(self, other, self))

    def __sub__(self, other):
        instance = self.__class__()
        for key, action in self._profiles.iteritems():
            if key not in other or other[key] != action:
                instance.insert(key, action)
        return instance

    @property
    def profiles(self):
        """Get a sorted dict of profiles"""
        return self._profiles

    def add(self, value):
        """Add profiles defines in value"""
        action = Actions.INSERT
        value = value.strip()
        if value and value.strip()[0] == '!':
            action = Actions.REMOVE
        self._profiles[self._proregex.sub("", value)] = action

    def insert(self, value, action=Actions.INSERT):
        """insert profile value with action"""
        self._profiles[value] = action

    def delete(self, value):
        """Remove profile from available ones"""
        del self._profiles[value]

    def extend(self, values):
        """extend profile with values"""
        # parse comma separated list of values
        if type(values) in (str, unicode):
            values = values.split(',')
        if values is not None:
            [self.add(prof) for prof in \
                 ifilter(lambda x: len(x) > 0, values)]


class ProfileFactory(NamedAbstractFactory):
    """Main Flow(Profiles) factory"""
    def __init__(self, *args, **kwargs):
        NamedAbstractFactory.__init__(self, *args, **kwargs)

    def flows(self):
        """Get a string list of available"""
        backends = self.backends.itervalues()
        return ",".join((backend.PROVIDES for backend in backends))


class MetaProfile(type):
    """Profile Metaclass"""
    def __init__(mcs, name, bases, dct):
        type.__init__(mcs, name, bases, dct)
        if name != "BaseProfile":
            log.profiles.info("'%s' profile flow registered", mcs.PROVIDES)
            ProfileFactory.get_instance().register(name, mcs)


class BaseProfile(object):
    """Base class to define Profile Flows"""

    __metaclass__ = MetaProfile

    PROVIDES = ""
    REGISTER = []

    def __init__(self, dispatch, profiles):
        self._dispatch = dispatch
        self._profiles = profiles
        # Register actions
        action = self._profiles.get(self.PROVIDES, Actions.INSERT)
        if action != Actions.REMOVE:
            #pylint: disable-msg=W0142
            [self.register(*line) for line in self.REGISTER]

    def register(self, event, state, action, view, where=None, animation=None):
        """Register a flow into Application Controller"""
        dispacher = self._dispatch
        if self._profiles[where] != Actions.REMOVE:
            dispacher.register(event, state, action, view, where, animation)

    #pylint: disable-msg=W0613
    @classmethod
    def can_handle(cls, dispatch, profiles):
        """Verify that this profile should be active"""
        return cls.PROVIDES in profiles

# Now its safe to register all flows with ProfileFactory. Import flows
# MUST be always declared AFTER BaseProfile and ProfileFactory
#pylint: disable-msg=W0611
from .import flows
